import React, {useState, useRef, useEffect} from 'react';
import {Button, Stack} from "react-bootstrap";
import 'bootstrap/dist/css/bootstrap.min.css';
import './App.css';
import * as d3 from 'd3'

function App() {
    const [enregistre, setEnregistre] = useState(false);
    const [text,setText] = useState("Les données ne sont pas enregistrées");
    const svgRef = useRef(null);


    // Équivalent à componentDidMount plus componentDidUpdate :
    useEffect(async () => {
        if (enregistre) {
            setText("Enregistrement en cours")
        } else {
            setText("Les données ne sont pas enregistrées")
        }
        // const d3 = await import('d3');
        const data = [{date: new Date("2007-04-23"), valeur: 20.24},
            {date: new Date("2007-04-25"), valeur: 15},
            {date: new Date("2007-04-27"), valeur: 17}];
        const hauteurRestante = document.documentElement.clientHeight - document.getElementById("selection").clientHeight - 20;
        const largeurRestante = document.getElementById("selection").clientWidth;
        let lineChart = LineChart(data, {
            x: d => d.date,
            y: d => d.valeur,
            yLabel: "↑ Daily valeur ($)",
            width: largeurRestante,
            height: hauteurRestante,
            color: "steelblue"
        })
        d3.select("svg").selectAll("*").remove();
        const svg =  d3.select(svgRef.current)
            .attr("width", largeurRestante)
            .attr("height", hauteurRestante);

        svg.append(() => lineChart);
        //   console.log(chart);
        // var svg = d3.select('#chart');
        // svg.append(chart);
        //  svg.
        console.log(enregistre);
    });

    return (
        <>
            <head>
                <title>controlleur capteur</title>
                <script src="https://unpkg.com/react/umd/react.production.min.js" crossOrigin/>
                <script type="text/javascript" src="./d3.js" charSet="utf-8"></script>
                <script type="text/javascript" src="./d3-line-chart.js" charSet="utf-8"></script>
            </head>
            <div id="selection">
                <h1>Lancer l'enregistrement des données des capteurs</h1>
                <Stack gap={2} className="col-md-5 mx-auto">
                    <p>{text}</p>
                    <Button
                        variant="outline-primary"
                        onClick={() => setEnregistre(!enregistre)}
                    >enregistrement
                    </Button>{' '}
                </Stack>
            </div>
            <div id="affichage">
                {<svg ref={svgRef} />}
            </div>
        </>
    );
}

// Copyright 2021 Observable, Inc.
// Released under the ISC license.
// https://observablehq.com/@d3/line-chart
function LineChart(data, {
    x = ([x]) => x, // given d in data, returns the (temporal) x-value
    y = ([, y]) => y, // given d in data, returns the (quantitative) y-value
    defined, // for gaps in data
    curve = d3.curveLinear, // method of interpolation between points
    marginTop = 20, // top margin, in pixels
    marginRight = 30, // right margin, in pixels
    marginBottom = 30, // bottom margin, in pixels
    marginLeft = 40, // left margin, in pixels
    width = 640, // outer width, in pixels
    height = 400, // outer height, in pixels
    xType = d3.scaleUtc, // the x-scale type
    xDomain, // [xmin, xmax]
    xRange = [marginLeft, width - marginRight], // [left, right]
    yType = d3.scaleLinear, // the y-scale type
    yDomain, // [ymin, ymax]
    yRange = [height - marginBottom, marginTop], // [bottom, top]
    yFormat, // a format specifier string for the y-axis
    yLabel, // a label for the y-axis
    color = "currentColor", // stroke color of line
    strokeLinecap = "round", // stroke line cap of the line
    strokeLinejoin = "round", // stroke line join of the line
    strokeWidth = 1.5, // stroke width of line, in pixels
    strokeOpacity = 1, // stroke opacity of line
} = {}) {
    // Compute values.
    const X = d3.map(data, x);
    const Y = d3.map(data, y);
    const I = d3.range(X.length);
    if (defined === undefined) defined = (d, i) => !isNaN(X[i]) && !isNaN(Y[i]);
    const D = d3.map(data, defined);

    // Compute default domains.
    if (xDomain === undefined) xDomain = d3.extent(X);
    if (yDomain === undefined) yDomain = [0, d3.max(Y)];

    // Construct scales and axes.
    const xScale = xType(xDomain, xRange);
    const yScale = yType(yDomain, yRange);
    const xAxis = d3.axisBottom(xScale).ticks(width / 80).tickSizeOuter(0);
    const yAxis = d3.axisLeft(yScale).ticks(height / 40, yFormat);

    // Construct a line generator.
    const line = d3.line()
        .defined(i => D[i])
        .curve(curve)
        .x(i => xScale(X[i]))
        .y(i => yScale(Y[i]));

    console.log(line(I));

    const svg = d3.create("svg")
        .attr("width", width)
        .attr("height", height)
        .attr("viewBox", [0, 0, width, height])
        .attr("style", "max-width: 100%; height: auto; height: intrinsic;");

    svg.append("g")
        .attr("transform", `translate(0,${height - marginBottom})`)
        .call(xAxis);

    svg.append("g")
        .attr("transform", `translate(${marginLeft},0)`)
        .call(yAxis)
        .call(g => g.select(".domain").remove())
        .call(g => g.selectAll(".tick line").clone()
            .attr("x2", width - marginLeft - marginRight)
            .attr("stroke-opacity", 0.1))
        .call(g => g.append("text")
            .attr("x", -marginLeft)
            .attr("y", 10)
            .attr("fill", "currentColor")
            .attr("text-anchor", "start")
            .text(yLabel));
    console.log(line(I))
    svg.append("path")
        .attr("fill", "none")
        .attr("stroke", color)
        .attr("stroke-width", strokeWidth)
        .attr("stroke-linecap", strokeLinecap)
        .attr("stroke-linejoin", strokeLinejoin)
        .attr("stroke-opacity", strokeOpacity)
        .attr("d", line(I));
    return svg.node();
}

export default App;